

class Status:
    NORMAL = 1000
    GOING_AWAY = 1001
    DATA_ERROR = 1002

    RESERVED = 1004
    RESERVED2 = 1005

    BAD_DATA = 1007
    POLICY_VIOLATION = 1008
    MESSAGE_TOO_LARGE = 1009
    EXTENSION_FAILED = 1010

    UNEXPECTED_CONDITION = 1011
